﻿using System;
using System.Runtime.InteropServices;

namespace ConsoleApplication1
{
    [StructLayout(LayoutKind.Auto)]
    class MyClassA
    {
        public int i = 1;
        public string s = "2";
        public double d = 2;
        public byte b = 3;
    }

    [StructLayout(LayoutKind.Sequential, Pack = 1)] 
    class MyClassB
    {
        public int i = 1;
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 16)]
        public string s = "2";
        public double d = 3;
        public byte b = 4;
    }

    [StructLayout(LayoutKind.Explicit, Pack = 1)]
    class MyClassC
    {
        [FieldOffset(0)]
        public int i = 1;
        [FieldOffset(4)]
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 16)]
        public string s = "2";
        [FieldOffset(4 + 16)]
        public double d = 3;
        [FieldOffset(4 + 16 + 8)]
        public byte b = 4;
    }

    class Program
    {
        [DllImport("Win32Project1.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern void fnWin32Project1(IntPtr cl);
        [DllImport("Win32Project1.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern void fnWin32Project2(IntPtr cl);

        unsafe static void Main(string[] args)
        {
            MyClassA var1 = new MyClassA();
            MyClassB var2 = new MyClassB();
            MyClassC var3 = new MyClassC();

            int sizeofA = 100; //Marshal.SizeOf(var1);
            int sizeofB = Marshal.SizeOf(var2);
            int sizeofC = Marshal.SizeOf(var3);

            IntPtr ptrA = Marshal.AllocCoTaskMem(sizeofA);
            // Marshal.StructureToPtr(var1, ptrA, false);

            IntPtr ptrB = Marshal.AllocCoTaskMem(sizeofB);
            Marshal.StructureToPtr(var2, ptrB, false);

            IntPtr ptrC = Marshal.AllocCoTaskMem(sizeofC);
            Marshal.StructureToPtr(var3, ptrC, false);

            fixed(double *pD1 = &var2.d)
            {
                IntPtr ptrPD = new IntPtr(pD1);
                Console.WriteLine("Address of MyClassB.d: " + ptrPD.ToString("x"));
            }

            fixed (int* pD2 = &var3.i)
            {
                IntPtr ptrPD = new IntPtr(pD2);
                Console.WriteLine("Address of MyClassC.i: " + ptrPD.ToString("x"));
            }

            fnWin32Project1(ptrB);
            fnWin32Project2(ptrC);


            Console.ReadLine();
        }
    }
}
